"""Commands to make entries for beancounter."""

import datetime
import typer
import csv
import requests  # type: ignore
import json
from datetime import datetime as dt
import logging
import rich

logging.basicConfig(
    filename="bean.log",
    level=logging.DEBUG,
    format="%(module)s %(funcName)s %(lineno)d %(levelname)s:: %(message)s",
)


app = typer.Typer()


def get_rate(date):
    """Returns the USDtoINR Exchange rate."""
    url = f"https://api.frankfurter.app/{date.strftime('%Y-%m-%d')}?amount=1&from=USD&to=INR"
    x = requests.get(url, timeout=5)
    logging.info(x)
    logging.info(x.text)
    j = json.loads(x.text)
    return j["rates"]["INR"]


@app.command()
def income(
    client: str,
    bill_date: datetime.datetime,
    bill_value: float,
    inr_to_india: float,
    cgst: float,
):
    """Add a new income."""
    rich.print(f"   Income:USD:{client} {bill_value} USD @ {get_rate(bill_date)} INR")
    rich.print(f"   Assets:INR:Cash:IDFC:Current {inr_to_india}")
    rich.print("   Assets:INR:ExchangeGain")
    rich.print(f"   Assets:INR:GST:ITC:CGST     {cgst} INR")
    rich.print(f"   Assets:INR:GST:ITC:SGST     {cgst} INR")


@app.command()
def upwork(file: str):
    """Processing Upworks Certificate of Earnings."""
    set = {
        "WHT": "Expenses:USD:Upwork:WHT",
        "Service Fee": "Expenses:USD:Upwork:ServiceFee",
        "Withdrawal Fee": "Expenses:USD:Upwork:WithdrawalFee",
        "Withdrawal": "Assets:INR:Cash:AccountReceivable:Upwork",
        "Hourly": "Income:USD:Upwork:Client",
        "Fixed Price": "Income:USD:Upwork:Client",
        "Bonus": "Income:USD:Upwork:Client",
        "Refund": "Income:USD:Upwork:Client",
        "Expense": "Income:USD:Upwork:Client",
    }
    with open(file) as csvfile:
        rows = csv.DictReader(csvfile)
        for r in rows:
            date = dt.strptime(r["Date"], "%b %d, %Y")  # noqa: DTZ007
            rich.print(
                date.strftime("%Y-%m-%d"),
                " * ",
                f'"Upworks {r["Team"]}"',
                f'"{r["Description"]}"',
            )

            conv = (
                f"@{get_rate(date)} INR"
                if set[r["Type"]]
                in [
                    "Income:USD:Upwork:Client",
                    "Assets:INR:Cash:AccountReceivable:Upwork",
                ]
                else ""
            )
            rich.print(f"    Assets:USD:Upwork {r['Amount']} USD {conv}")
            rich.print(f"    {set[r['Type']]} \n")


@app.command()
def razorpay(file: str):
    """Compile Razorpay records."""
    with open(file) as csvfile:
        rows = csv.DictReader(csvfile)
        for r in rows:
            date = dt.strptime(r["Date"], "%d/%m/%Y")  # noqa: DTZ007
            if r["Status"] == "Success":
                rich.print(
                    date.strftime("%Y-%m-%d"),
                    f'* "{r["Emp Name"]} {r["Type"]}"\n',
                    "   Assets:INR:Cash:RazorPay",
                )
                if r["Credit"] != "":
                    rich.print(f"    Assets:INR:Cash:IDFC:Current {r['Credit']} INR")
                else:
                    rich.print(f"    Expenses:INR:Salary {r['Debit']} INR")



"""Command Line Interface."""

# import pkg_resources
import subprocess
import copier
import typer
import os
import yaml  # type: ignore
from xdg_base_dirs import xdg_config_home
from .beancounter import app as bean  # type:ignore

app = typer.Typer()
app.add_typer(bean, name="bean")


@app.command()
def run() -> None:
    """Run command."""
    print("Hello World")


@app.command()
def venv() -> None:
    """Creates a virtual env file."""
    subprocess.run(["python3", "-m", "venv", "venv"], check=False)


@app.command()
def ip(name: str, org: str = "dyu.yaml") -> None:
    """Creates an IP folder layout."""
    data = read_config(org)
    print(f"DATA={data}")
    copier.run_copy("gh:dyu-copier/hdl_unit", name, data=data)


@app.command()
def cocotbext(name: str, org: str = "dyu.yaml") -> None:
    """Creates an cocotbext plugin folder layout."""
    copier.run_copy("gh:dyu-copier/cocotbext", name, data=read_config(org))


@app.command()
def peakrdl(name: str, org: str = "dyu.yaml") -> None:
    """Creates an peakrdl plugin folder layout."""
    copier.run_copy("gh:dyu-copier/peakrdl", name, data=read_config(org))


@app.command()
def plan(configfile: str, org: str = "dyu.yaml") -> None:
    """Creates a plan using taskJuggler."""
    # template = pkg_resources.resource_filename("dyu", "template/plan.tji")
    raise NotImplementedError("This feature is not implemented")


def read_config(file="config.yml"):
    """Reads config file at ~/.config/dyu/config.yml and returns the value."""
    cfg_path = os.path.join(xdg_config_home(), "dyu")
    cfg_file = os.path.join(cfg_path, file)
    if not os.path.exists(cfg_path):
        os.mkdir(cfg_path)
    if not os.path.exists(cfg_file):
        with open(cfg_file, "w") as cfg:
            print(
                "This is a one time setup of your configuration data for all projects"
            )
            name = typer.prompt("Your Name")
            org = typer.prompt("Your Org")
            email = typer.prompt("Your Email ID")
            yaml.dump({"author": name, "org": org, "email": email}, cfg)
    with open(cfg_file) as cfg:
        data = yaml.safe_load(cfg)
    print(data)
    return data


@app.callback(no_args_is_help=True)
def main() -> None:
    """CLI for Dyumnin supertool."""


typer_click_object = typer.main.get_command(app)
